﻿using System;
using System.Collections.Generic;
using System.Text;

public class Person : ITuple<string, string, DateTime>, IRandomAccessTuple, INamedAccessTuple, IArrayAccessTuple {

    private string m_FirstName;
    private string m_LastName;
    private DateTime m_Born;

    public Person(string sFirstName, string sLastName, DateTime born) {
        m_FirstName = sFirstName;
        m_LastName = sLastName;
        m_Born = born;

    }

    public string FirstName {
        get {
            return m_FirstName;
        }
    }

    public string LastName {
        get {
            return m_LastName;
        }
    }

    public DateTime Born{
        get{
            return m_Born;
        }
    }

    public override string ToString() {
        return "Person {FirstName: " + m_FirstName + "; LastName: " + m_LastName + "; Born: " + m_Born + "}";
    }

    public override bool Equals(object obj) {
        if(obj != null && obj.GetType().Equals(this.GetType())) {
            Person that = (Person) obj;
            return 0 == string.Compare(this.m_FirstName, that.m_FirstName)
                && 0 == string.Compare(this.m_LastName, that.m_LastName);
        } else {
            return false;
        }
    }

    public override int GetHashCode() {
        return (m_FirstName.GetHashCode() ^ m_LastName.GetHashCode());
    }

    #region "ITuple<T1, T2>"
    /* private implement */ int ITuple.ElementCount {
        get {
            return 2;
        }
    }
    /* private implement */ string ITuple<string>.Value1{
        get{
            return m_FirstName;
        }
        set{
            m_FirstName = value;
        }
    }
    /* private implement */ string ITuple<string, string>.Value2{
        get{
            return m_LastName;
        }
        set{
            m_LastName = value;
        }
    }
    /* private implement */ DateTime ITuple<string, string, DateTime>.Value3{
        get{
            return m_Born;
        }
        set{
            m_Born = value;
        }
    }
    #endregion

    #region "IRandomAccessTuple"
    /* private implement */ object IRandomAccessTuple.GetElement(int index) {
        switch(index) {
            case 0:
                return m_FirstName;
            case 1:
                return m_LastName;
            case 2:
                return m_Born;
            default:
                throw new IndexOutOfRangeException();
        }
    }
    /* private implement */ void IRandomAccessTuple.SetElement(int index, object value){
        switch(index){
            case 0:
                m_FirstName = (string) value;
                break;
            case 1:
                m_LastName = (string) value;
                break;
            case 2:
                m_Born = (DateTime) value;
                break;
            default:
                throw new IndexOutOfRangeException();
        }
    }
    #endregion

    #region "INamedAccessTuple"
    /* private implement */ string[] INamedAccessTuple.GetNames(){
        return new string[] {"FirstName", "LastName", "Born"};
    }
    /* private implement */ object INamedAccessTuple.GetElement(string sElementName) {
        switch(sElementName) {
            case "FirstName":
                return m_FirstName;
            case "LastName":
                return m_LastName;
            case "Born":
                return m_Born;
            default:
                throw new ArgumentException();
        }
    }
    /* private implement */ void INamedAccessTuple.SetElement(string sElementName, object value){
        switch(sElementName){
            case "FirstName":
                m_FirstName = (string) value;
                break;
            case "LastName":
                m_LastName = (string) value;
                break;
            case "Born":
                m_Born = (DateTime) value;
                break;
            default:
                throw new ArgumentException();
        }
    }
    #endregion

    #region "IArrayAccessTuple"
    /* private implement */ object[] IArrayAccessTuple.GetElements() {
        return new object[] {m_FirstName, m_LastName, m_Born};
    }
    /* private implement */ void IArrayAccessTuple.SetElements(params object[] values){
        if(values.Length > 0){
            m_FirstName = (string) values[0];
        }else if(values.Length > 1){
            m_LastName = (string) values[1];
        }else if(values.Length > 2){
            m_Born = (DateTime) values[2];
        }
    }
    #endregion 

}